//+------------------------------------------------------------------+
//|                                         FX5_SelfAdjustingRSI.mq4 |
//|                                                              FX5 |
//|                                                    hazem@uk2.net |
//+------------------------------------------------------------------+
#property copyright "FX5"
#property link      "hazem@uk2.net"
#property link "Histogram modification by omelette"
//----
#property indicator_buffers 4
#property indicator_separate_window
#property indicator_minimum -50
#property indicator_maximum 50
#property indicator_level1  30
#property indicator_level2  0
#property indicator_level3  -30
//---- input parameters
extern int    rsiPeriod = 14;
extern double diviation = 2;
extern bool   MA_Method = true;
extern color  rsiColor = Blue;
extern color  overBoughtColor = Red;
extern color  overSoldColor = Green;
//---- buffers
double RSIdiff[],rsi[];
double upperBorder[];
double lowerBorder[];
double absDiviation[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0, DRAW_HISTOGRAM,STYLE_SOLID,3,Yellow);
   SetIndexStyle(1, DRAW_LINE, EMPTY, EMPTY, rsiColor);
   SetIndexStyle(2, DRAW_LINE, EMPTY, EMPTY, overBoughtColor);
   SetIndexStyle(3, DRAW_LINE, EMPTY, EMPTY, overSoldColor);
//----   
   SetIndexBuffer(0,RSIdiff);
   SetIndexBuffer(1, rsi);
   SetIndexBuffer(2, upperBorder);   
   SetIndexBuffer(3, lowerBorder);
//----   
   SetIndexDrawBegin(0, rsiPeriod);
   SetIndexDrawBegin(1, rsiPeriod);
   SetIndexDrawBegin(2, rsiPeriod * 2);
   SetIndexDrawBegin(3, rsiPeriod * 2);
//----   
   SetIndexLabel(0, "RSIdiff");
   SetIndexLabel(1, "RSI");
   SetIndexLabel(2, "OverBought");
   SetIndexLabel(3, "OverSold");   
//----
   IndicatorShortName("FX5_SelfAdjustingRSI( " + rsiPeriod + " )");
   IndicatorBuffers(5);
   SetIndexBuffer(4, absDiviation);      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int countedBars = IndicatorCounted();
//----   
   if(MA_Method == true)
       MovingAverageMethod(countedBars);
   else
       StandardDeviationMethod(countedBars);
//----         
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void MovingAverageMethod(int countedBars)
  {
   if(countedBars < 0)
      countedBars = 0;
//----      
   for(int i = Bars - countedBars; i >= 0; i--)
     {     
       rsi[i] = iRSI(NULL, 0, rsiPeriod, PRICE_CLOSE, i) - 50;
       double smoothedRSI = GetSmoothedRSI(i);
       absDiviation[i] = MathAbs(rsi[i] - smoothedRSI);
       double kDiviation = diviation * GetAbsDiviationAverage(i);
       upperBorder[i] = kDiviation;
       lowerBorder[i] = -kDiviation;
       if (rsi[i] > 0)
        RSIdiff[i] = MathMax(0,rsi[i]-upperBorder[i]);
       else if (rsi[i] < 0)
        RSIdiff[i] = MathMin(0,-lowerBorder[i]+rsi[i]);
     }            
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void StandardDeviationMethod(int countedBars)
  {
   if(countedBars < 0)
       countedBars = 0;
//----
   for(int i = Bars - countedBars; i >= 0; i--)
     {     
       rsi[i] = iRSI(NULL, 0, rsiPeriod, PRICE_CLOSE, i);
       double rsiDiviation = iStdDevOnArray(rsi, 0, rsiPeriod, 0, 
                                            MODE_SMA, i);
       double kDiviation = diviation * rsiDiviation;
       upperBorder[i] = 50 + kDiviation;
       lowerBorder[i] = 50 - kDiviation;      
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetSmoothedRSI(int shift)
  {
   double sum = 0;
   for(int i = shift; i < shift + rsiPeriod; i++)
     {
       sum += rsi[i];
     }
   return(sum / rsiPeriod);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetAbsDiviationAverage(int shift)
  {
   double sum = 0;
   for(int i = shift; i < shift + rsiPeriod; i++)
     {
       sum += absDiviation[i];
     }
   return(sum / rsiPeriod);
  }
//+------------------------------------------------------------------+

